# django imports
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist

# portlets imports
from portlets.models import PortletAssignment
from portlets.models import PortletBlocking
from portlets.models import PortletRegistration
from portlets.models import Slot

def get_slots(obj):
    """Returns slot with all assigned portlets as dict.
    """
    portlet_types = get_registered_portlets()
    ct = ContentType.objects.get_for_model(obj)

    # Stores whether at least one slot has at least one portlet.
    has_portlets = False

    items = []
    for slot in Slot.objects.all():
        temp = []
        for pa in PortletAssignment.objects.filter(
            slot=slot, content_id=obj.id, content_type=ct.id):
            has_portlets = True
            temp.append({
                "pa_id" : pa.id,
                "title" : pa.portlet.title,
                "type" : portlet_types.get(pa.portlet.__class__.__name__.lower(), ""),
            })

        items.append({
            "id"   : slot.id,
            "name" : slot.name,
            "is_blocked" : is_blocked(obj, slot),
            "portlets" : temp,
        })

    return {
        "has_portlets" : has_portlets,
        "items" : items
    }

def is_blocked(obj, slot):
    """Returns True if for the given object the given slot is blocked.
    """
    ct = ContentType.objects.get_for_model(obj)
    try:
        pb = PortletBlocking.objects.get(
            slot=slot, content_type=ct.id, content_id=obj.id)
    except PortletBlocking.DoesNotExist:
        return False

    return True

def has_portlets(slot, obj):
    """Returns True if the given obj has portlets for given slot.
    """
    while obj:
        if len(get_portlets(slot, obj)) > 0:
            return True
        if is_blocked(obj, slot):
            break
        obj = obj.get_parent_for_portlets()

    return False

def get_portlets(slot, obj):
    """Returns portlet objs for a given slot and obj (content object).
    """
    ctype = ContentType.objects.get_for_model(obj)
    try:
        slot = Slot.objects.get(id=slot.id)
    except ObjectDoesNotExist:
        portlet_assignments = []
    else:
        portlet_assignments = PortletAssignment.objects.filter(
            slot=slot, content_type=ctype.id, content_id=obj.id).order_by("position")

    portlets = []
    for portlet_assignment in portlet_assignments:
        portlets.append(portlet_assignment.portlet)

    return portlets

def get_registered_portlets():
    """Returns registered portlet types as dict.
    """
    portlet_types = {}
    for pr in PortletRegistration.objects.all():
        portlet_types[pr.type] = pr.name

    return portlet_types

def register_portlet(obj, name):
    """Registers a portlet.
    """
    type = obj.__name__.lower()
    try:
        PortletRegistration.objects.create(type=type, name=name)
    except:
        pass

def unregister_portlet(obj):
    """Unregisters portlet with given type
    """
    type = obj.__name__.lower()
    try:
        pr = PortletRegistration.objects.get(type=type)
    except PortletRegistration.DoesNotExist:
        pass
    else:
        pr.delete()


from south.db import db
from django.db import models
from portlets.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'PortletRegistration.active'
        db.add_column('portlets_portletregistration', 'active', models.BooleanField(default=True))
        
    def backwards(self, orm):
        
        # Deleting field 'PortletRegistration.active'
        db.delete_column('portlets_portletregistration', 'active')
        
    models = {
        'portlets.portletblocking': {
            'Meta': {'unique_together': '["slot","content_id","content_type"]'},
            'content_id': ('models.PositiveIntegerField', [], {}),
            'content_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"pb_content"'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'slot': ('models.ForeignKey', ["orm['portlets.Slot']"], {})
        },
        'portlets.slot': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', [], {'max_length': '50'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label','model'),)", 'db_table': "'django_content_type'"},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'portlets.portletassignment': {
            'Meta': {'ordering': '["position"]'},
            'content_id': ('models.PositiveIntegerField', [], {}),
            'content_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"pa_content"'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'portlet_id': ('models.PositiveIntegerField', [], {}),
            'portlet_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"pa_portlets"'}),
            'position': ('models.PositiveSmallIntegerField', [], {'default': '999'}),
            'slot': ('models.ForeignKey', ["orm['portlets.Slot']"], {})
        },
        'portlets.portletregistration': {
            'Meta': {'ordering': '("name",)'},
            'active': ('models.BooleanField', [], {'default': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'type': ('models.CharField', [], {'unique': 'True', 'max_length': '30'})
        }
    }
    
    complete_apps = ['portlets']



from django.contrib import admin

from portlets.models import PortletAssignment
from portlets.models import PortletBlocking
from portlets.models import PortletRegistration
from portlets.models import Slot

admin.site.register(PortletAssignment)
admin.site.register(PortletRegistration)
admin.site.register(PortletBlocking)
admin.site.register(Slot)


# django imports
from django import template
from django.contrib.contenttypes.models import ContentType

# portlets imports
import portlets.utils
from portlets.models import PortletBlocking
from portlets.models import Slot

register = template.Library()

@register.inclusion_tag('portlets/portlet_slot.html', takes_context=True)
def portlet_slot(context, slot_name, instance=None):
    """Returns the portlets for given slot and instance. If the instance
    implements the ``get_parent_for_portlets`` method the portlets of the
    parent of the instance are also added.
    """
    if instance is None:
        return { "portlets" : [] }

    try:
        slot = Slot.objects.get(name=slot_name)
    except Slot.DoesNotExist:
        return { "portlets" : [] }

    # Get portlets for given instance
    temp = portlets.utils.get_portlets(slot, instance)

    # Get inherited portlets
    try:
        instance.get_parent_for_portlets()
    except AttributeError:
        instance = None

    while instance:
        # If the portlets are blocked no portlets should be added
        if portlets.utils.is_blocked(instance, slot):
            break

        # If the instance has no get_parent_for_portlets, there are no portlets
        try:
            instance = instance.get_parent_for_portlets()
        except AttributeError:
            break

        # If there is no parent for portlets, there are no portlets to add
        if instance is None:
            break

        parent_portlets = portlets.utils.get_portlets(slot, instance)
        parent_portlets.reverse()
        for p in parent_portlets:
            if p not in temp:
                temp.insert(0, p)

    rendered_portlets = []
    for portlet in temp:
        rendered_portlets.append(portlet.render(context))

    return { "portlets" : rendered_portlets }


# django imports
from django import forms
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

class Portlet(models.Model):
    """Base portlet. All portlets should inherit from this class.
    """
    title = models.CharField(blank=True, max_length=100)

    class Meta:
        abstract = True

    def render(self):
        """Renders the portlet as html. Have to be implemented by sub classes.
        """
        raise NotImplemented

    def form(self, **kwargs):
        """returns the form for the portlet. Have to be implemented by sub
        classes.
        """
        raise NotImplemented

class PortletAssignment(models.Model):
    """Assigns portlets to slots and content.
    """
    slot = models.ForeignKey("Slot")

    content_type = models.ForeignKey(ContentType, related_name="pa_content")
    content_id = models.PositiveIntegerField()
    content = generic.GenericForeignKey('content_type', 'content_id')

    portlet_type = models.ForeignKey(ContentType, related_name="pa_portlets")
    portlet_id = models.PositiveIntegerField()
    portlet = generic.GenericForeignKey('portlet_type', 'portlet_id')

    position = models.PositiveSmallIntegerField(default=999)

    class Meta:
        ordering = ["position"]
        verbose_name_plural = "Portlet assignments"

    def __unicode__(self):
        try:
            return "%s (%s)" % (self.portlet.title, self.slot.name)
        except AttributeError:
            return ""

class PortletBlocking(models.Model):
    """Blocks portlets for given slot and content object.
    """
    slot = models.ForeignKey("Slot")

    content_type = models.ForeignKey(ContentType, related_name="pb_content")
    content_id = models.PositiveIntegerField()
    content = generic.GenericForeignKey('content_type', 'content_id')

    class Meta:
        unique_together = ["slot", "content_id", "content_type"]

    def __unicode__(self):
        try:
            return "%s (%s)" % (self.content.title, self.slot.name)
        except AttributeError:
            return ""

class PortletRegistration(models.Model):
    """Registered portlets. These are provided to be added to customer.

    Parameters:

        * type:
            The type of the portlet. This must be the exactly class name of
            the portlet, e.g. TextPortlet

        * name
            The name of the portlet. This is displayed to the user.
            
        * active
            If true the portlet will be provided to assign to content object
     """
    type = models.CharField(max_length=30, unique=True)
    name = models.CharField(max_length=50, unique=True)
    active = models.BooleanField(default=True)

    class Meta:
        ordering = ("name", )

    def __unicode__(self):
        return "%s %s" % (self.type, self.name)


class Slot(models.Model):
    """Slots are places to which portlets can be assigned.
    """
    name = models.CharField(max_length=50)

    def __unicode__(self):
        return self.name